package org.ananas.hc;

import java.text.MessageFormat;

public class QName
{
   public final static int ROOT = 1,
                           ELEMENT = 2,
                           ATTRIBUTE = 3,
                           JOKER = 4,
                           END_MARK = 5;
   protected final static int LAST_QNAME_TYPE = END_MARK;
   protected String namespaceURI,
                    localName;
   protected int type;

   public QName(int type,String namespaceURI,String localName)
   {
      setType(type);
      if(type == ROOT)
      {
         this.namespaceURI = "".intern();
         this.localName = "/".intern();
      }
      else if(type == END_MARK)
      {
         this.namespaceURI = "".intern();
         this.localName = "(#)";
      }
      else if(type == JOKER)
      {
         this.namespaceURI = "".intern();
         this.localName = "*".intern();
      }
      else
      {
         if(localName == null && type < LAST_QNAME_TYPE)
            throw new IllegalArgumentException("local name is null");
         this.namespaceURI = namespaceURI == null ? "".intern() : namespaceURI.intern();
         this.localName = localName == null ? "".intern() : localName.intern();
      }
   }

   public QName(int type)
   {
      this(type,null,null);
   }

   public String getNamespaceURI()
   {
      return namespaceURI;
   }

   public String getLocalName()
   {
      return localName;
   }

   protected void setType(int type)
   {
      if(type < ROOT || type > LAST_QNAME_TYPE)
         throw new IllegalArgumentException("unknown type");
      else
         this.type = type;
   }

   public int getType()
   {
      return type;
   }

   public boolean equals(Object object)
   {
      if(object instanceof QName)
         return matches((QName)object);
      else
         return false;
   }

   public boolean matches(QName qname)
   {
      if(type == JOKER)
         return true;
      else
         return namespaceURI == qname.namespaceURI &&
                localName == qname.localName &&
                type == qname.type;
   }

   public int hashCode()
   {
      // don't worry about the type, it's uncommon enough that
      // elements and attributes have the same name that it should
      // not result in too many collisions
      return namespaceURI.hashCode() ^ localName.hashCode();
   }

   public String toString()
   {
      String pattern = null;
      switch(type)
      {
         case ROOT:
            pattern = "(/)";
            break;
         case ATTRIBUTE:
            pattern = "'{'{0}'}'@{1}";
            break;
         case ELEMENT:
            pattern = "'{'{0}'}'{1}";
            break;
         case JOKER:
            return "(*)";
         case END_MARK:
            return "(#)";
         default:
            throw new NotImplementedException("unknown type");
      }
      return MessageFormat.format(pattern,
                                  new Object[] { namespaceURI, localName });
   }
}
