package org.ananas.hc;

import java.util.Stack;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

public class XPathHandler
   extends DefaultHandler
{
   protected HCTables tables;
   protected int[] states;
   protected Stack stack;

   public XPathHandler(HCHandler handler)
      throws HCException
   {
      try
      {
         Class handlerClass = handler.getClass(),
               tablesClass = handlerClass.forName(handlerClass.getName() +
                                                  HCTables.CLASS_SUFFIX);
         tables = (HCTables)tablesClass.newInstance();
         tables.setHCHandler(handler);
      }
      catch(ClassNotFoundException e)
      {
         throw new HCException(e);
      }
      catch(IllegalAccessException e)
      {
         throw new HCException(e);
      }
      catch(InstantiationException e)
      {
         throw new HCException(e);
      }
   }

   public void startDocument()
      throws SAXException
   {
      stack = new Stack();
      QName qname = new QName(QName.ROOT);
      states = new int[tables.getCount()];
      for(int i = 0;i < tables.getCount();i++)
      {
         states[i] = tables.move(i,qname,-1);
         tables.acceptStartEvent(i,states[i],qname,null);
      }
   }

   public void startElement(String namespaceURI,
                            String localName,
                            String qualifiedName,
                            Attributes atts)
      throws SAXException
   {
      stack.push(states);
      QName qname = new QName(QName.ELEMENT,namespaceURI,localName);
      int[] cstates = states;
      states = new int[tables.getCount()];
      for(int i = 0;i < tables.getCount();i++)
      {
         if(tables.isAcceptingState(i,cstates[i]))
            states[i] = tables.move(i,qname,-1);
         else
            states[i] = tables.move(i,qname,cstates[i]);
         tables.acceptStartEvent(i,states[i],qname,atts);
      }
   }

   public void characters(char[] ch,
                          int start,
                          int length)
      throws SAXException
   {
      for(int i = 0;i < tables.getCount();i++)
         tables.acceptCharactersEvent(i,states[i],ch,start,length);
   }

   public void endElement(String namespaceURI,
                          String localName,
                          String qualifiedName)
      throws SAXException
   {
      QName qname = new QName(QName.ELEMENT,namespaceURI,localName);
      for(int i = 0;i < tables.getCount();i++)
         tables.acceptEndEvent(i,states[i],qname);
      states = (int[])stack.pop();
   }

   public void endDocument()
      throws SAXException
   {
      QName qname = new QName(QName.ROOT);
      for(int i = 0;i < tables.getCount();i++)
         tables.acceptEndEvent(i,states[i],qname);
   }
}
