/**
 * Default implementation for the messager. It prints most
 * error messages on a PrintWriter. See each method description
 * for more details. It flushes the writer after each writing operation.
 *
 * @author <a href="http://www.marchal.com">Beno&#238;t Marchal</a>
 */

// Copyright 2001-2002, Benot Marchal.

package org.ananas.hc.compiler;

import java.io.PrintWriter;
import java.io.OutputStreamWriter;

public class DefaultMessenger
   implements Messenger
{
   /**
    * PrintWriter this messager outputs to
    */
   protected PrintWriter writer;

   /**
    * Creates a DefaultMessenger that prints on the console.
    */
   public DefaultMessenger()
   {
      writer = new PrintWriter(new OutputStreamWriter(System.out));
   }

   /**
    * Creates a DefaultMessenger that prints on the given writer.
    * @param writer writer to use for output
    */
   public DefaultMessenger(PrintWriter writer)
   {
      this.writer = writer;
   }

   /**
    * Writes the exception's message and throws the exception.
    * @param e exception describing the error
    */
   public void error(CompilerException e)
      throws CompilerException
   {
      throw e;
   }

   /**
    * Writes the exception's message and throws the exception.
    * @param e exception describing the error
    */
   public void fatal(CompilerException e)
      throws CompilerException
   {
      throw e;
   }

   /**
    * Writes the warning message on the writer. Does not throw the exception.
    * @param e exception describing the error
    */
   public void warning(CompilerException e)
   {
      info(e.getLocalizedMessage());
   }

   /**
    * Writes the message on the writer.
    * @param msg exception describing the error
    */
   public void info(String msg)
   {
      writer.println(msg);
      writer.flush();
   }
}
