package org.ananas.hc.compiler;

import java.io.File;
import java.io.FileWriter;
import java.util.ArrayList;
import java.io.PrintWriter;
import java.io.IOException;
import org.ananas.hc.QName;
import org.ananas.hc.HCTables;
import org.ananas.hc.MessageStore;
import org.xml.sax.helpers.NamespaceSupport;
import org.ananas.hc.NotImplementedException;

// BUGBUG -- this is insane: in this setup, there's only
// one state per XPath so you need not have such a complicated
// acceptXXX() method
// you'd be happy with a simpler thing that says, it's accepting
// for XPath X, OKay. Here's what happens.

public class HCTablesGenerator
{
   protected MessageStore message;
   protected File destdir;
   protected DFAFactory factory;
   protected Messenger messenger;

   public HCTablesGenerator(MessageStore message,
                            Messenger messenger,
                            File destdir)
   {
      this.message = message;
      this.messenger = messenger;
      this.destdir = destdir;
      factory = new DFAFactory();
   }

   public void generateHCTables(HandlerInfo handler)
      throws CompilerException
   {
      messenger.info(message.getMessage("Compiling",handler.getName()));
      try
      {
         DFATable[] tables = compileDFA(handler);
         writeHCTables(handler,tables);
      }
      catch(IOException e)
      {
         error("IOException",e.getLocalizedMessage());
      }
   }

   protected DFATable[] compileDFA(HandlerInfo handler)
      throws CompilerException
   {
      XPathParser parser = new XPathParser(handler.getNamespaceSupport(),message);
      MethodInfo[] methods = handler.getMethods();
      ArrayList array = new ArrayList();
      for(int i = 0;i < methods.length;i++)
      {
         String[] xpaths = methods[i].getXPaths();
         for(int j = 0;j < xpaths.length;j++)
         {
            XPathNode node = parser.axpath(xpaths[j],i,methods[i]);
            if(node != null)
               array.add(factory.createDFA(node));
         }
      }
      DFATable[] tables = new DFATable[array.size()];
      return (DFATable[])array.toArray(tables);
   }

   protected File filename(HandlerInfo handler)
   {
      String classname = handler.getName() + HCTables.CLASS_SUFFIX;
      classname = classname.replace('.','/');
      classname += ".java";
      File file = new File(destdir,classname),
           parent = file.getParentFile();
      if(parent != null && !parent.exists())
         parent.mkdirs();
      return file;
   }

   protected String format(int a)
   {
      String st = Integer.toString(a);
      StringBuffer buffer = new StringBuffer();
      for(int i = 0;i < 4 - st.length();i++)
         buffer.append(' ');
      buffer.append(st);
      return buffer.toString();
   }

   protected String nameOfType(int type)
   {
      switch(type)
      {
         case QName.ROOT:
            return "QName.ROOT";
         case QName.ELEMENT:
            return "QName.ELEMENT";
         case QName.ATTRIBUTE:
            return "QName.ATTRIBUTE";
         case QName.JOKER:
            return "QName.JOKER";
         case QName.END_MARK:
            return "QName.END_MARK";
         default:
            throw new NotImplementedException("Unknown value for QName.getType()");
      }
   }

   protected final void error(String key,Object argument)
      throws CompilerException
   {
      messenger.error(new CompilerException(message.getMessage(key,argument)));
   }

   protected void writeHCTables(HandlerInfo handler,DFATable[] tables)
      throws CompilerException, IOException
   {
      File file = filename(handler);
      PrintWriter writer = new PrintWriter(new FileWriter(file));
      try
      {
         writer.println("// Do NOT edit. This file was generated automatically.");
         writer.println();
         if(handler.getPackageName() != null)
         {
            writer.print("package ");
            writer.print(handler.getPackageName());
            writer.println(';');
            writer.println();
         }
         writer.println("import java.util.Map;");
         writer.println("import java.util.HashMap;");
         writer.println("import org.ananas.hc.QName;");
         writer.println("import org.ananas.hc.HCTables;");
         writer.println("import org.xml.sax.Attributes;");
         writer.println("import org.ananas.hc.HCHandler;");
         writer.println("import org.xml.sax.SAXException;");
         writer.println("import org.ananas.hc.HCException;");
         writer.println();
         writer.println("/**");
         writer.println(" * HC unlocks the fun in SAX parsing.<br>");
         writer.println(" * A project developed for the 'Working XML' column at developerWorks.");
         writer.println(" * @see <a href='http://www.ibm.com/developerWorks'>developerWorks: a worldwide resource for developers</a>");
         writer.println(" * @see <a href='http://www.ananas.org'>HC and other projects at ananas.org</a>");
         writer.println(" * @see <a href='http://www.marchal.com'>XML books, articles by Beno&#238;t Marchal</a>");
         writer.println(" */");
         writer.println();
         writer.print("public final class ");
         writer.print(handler.getClassName());
         writer.println(HCTables.CLASS_SUFFIX);
         writer.println("   implements HCTables");
         writer.println('{');
         writer.print("   private ");
         writer.print(handler.getName());
         writer.println(" handler;");
         writer.println("   private static final int[][][] dtran =");
         writer.println("   {");
         for(int i = 0;i < tables.length;i++)
         {
            writer.println("      {");
            for(int j = 0;j < tables[i].dtran.length;j++)
            {
               writer.print("         {");
               for(int k = 0;k < tables[i].dtran[j].length;k++)
               {
                  if(k != 0)
                     writer.print(',');
                  writer.print(format(tables[i].dtran[j][k]));
               }
               writer.println(" },");
            }
            writer.println("      },");
         }
         writer.println("   };");
         writer.println("   private static final boolean[][] astates =");
         writer.println("   {");
         for(int i = 0;i < tables.length;i++)
         {
            writer.println("      {");
            writer.print("         ");
            for(int j = 0;j < tables[i].astate.length;j++)
            {
               if(j != 0)
                  writer.print(',');
               writer.print(tables[i].astate[j] != null ? "true" : "false");
            }
            writer.println();
            writer.println("      },");
         }
         writer.println("   };");
         writer.println("   private static Map[] symbols = null;");
         writer.println();
         writer.print("   public ");
         writer.print(handler.getClassName());
         writer.print(HCTables.CLASS_SUFFIX);
         writer.println("()");
         writer.println("   {");
         writer.println("      this.handler = handler;");
         writer.println("      synchronized(dtran)");
         writer.println("      {");
         writer.println("         if(symbols == null)");
         writer.println("         {");
         writer.print("            symbols = new HashMap[");
         writer.print(tables.length);
         writer.println("];");
         for(int i = 0;i < tables.length;i++)
         {
            writer.print("            symbols[");
            writer.print(i);
            writer.println("] = new HashMap();");
            for(int j = 0;j < tables[i].qnames.length;j++)
            {
               writer.print("            symbols[");
               writer.print(i);
               writer.print("].put(new QName(");
               int type = tables[i].qnames[j].getType();
               writer.print(nameOfType(type));
               if(tables[i].qnames[j].getLocalName() != null &&
                  (type != QName.ROOT && type != QName.END_MARK &&
                   type != QName.JOKER))
               {
                  writer.print(',');
                  if(tables[i].qnames[j].getNamespaceURI() == null)
                     writer.print("null");
                  else
                  {
                     writer.print('"');
                     writer.print(tables[i].qnames[j].getNamespaceURI());
                     writer.print('"');
                  }
                  writer.print(",\"");
                  writer.print(tables[i].qnames[j].getLocalName());
                  writer.print('"');
               }
               writer.print("),new Integer(");
               writer.print(Integer.toString(j));
               writer.println("));");
            }
         }
         writer.println("         }");
         writer.println("      }");
         writer.println("   }");
         writer.println();
         writer.println("   public void setHCHandler(HCHandler handler)");
         writer.println("   {");
         writer.print("      this.handler = (");
         writer.print(handler.getClassName());
         writer.println(")handler;");
         writer.println("   }");
         writer.println();
         writer.println("   public int getCount()");
         writer.println("   {");
         writer.print("      return ");
         writer.print(tables.length);
         writer.println(";");
         writer.println("   }");
         writer.println();
         writer.println("   public int move(int xpath,QName qname,int state)");
         writer.println("   {");
         writer.println("      Integer key = (Integer)symbols[xpath].get(qname);");
         writer.println("      if(key != null)");
         writer.println("         return dtran[xpath][state < 0 ? 0 : state][key.intValue()];");
         writer.println("      else");
         writer.println("         return -1;");
         writer.println("   }");
         writer.println();
         writer.println("   public boolean isAcceptingState(int xpath,int state)");
         writer.println("   {");
         writer.println("      return state == -1 ? false : astates[xpath][state];");
         writer.println("   }");
         writer.println();
         writer.println("   public void acceptStartEvent(int xpath,int state,QName qname,Attributes atts)");
         writer.println("      throws SAXException");
         writer.println("   {");
         writer.println("      try");
         writer.println("      {");
         writer.println("         switch(xpath)");
         writer.println("         {");
         for(int i = 0;i < tables.length;i++)
         {
            writer.print("            case ");
            writer.print(i);
            writer.println(':');
            writer.println("               switch(state)");
            writer.println("               {");
            for(int j = 0;j < tables[i].astate.length;j++)
            {
               MethodInfo method = (MethodInfo)tables[i].astate[j];
               if(method != null &&
                  !method.getName().startsWith("characters") &&
                  !method.getName().startsWith("content") &&
                  !method.getName().startsWith("end") &&
                  !method.getName().startsWith("close"))
               {
                  writer.print("                  case ");
                  writer.print(Integer.toString(j));
                  writer.println(':');
                  writer.print("                     handler.");
                  writer.print(method.getName());
                  writer.print('(');
                  String[][] params = method.getParameters();
                  for(int k = 0;k < params.length;k++)
                  {
                     if(k != 0)
                        writer.print(',');
                     if(params[k][0].equals("org.ananas.hc.QName"))
                        writer.print("qname");
                     else if(params[k][0].equals("org.xml.sax.Attributes"))
                        writer.print("atts");
                     else if(params[k][0].equals("java.lang.String"))
                     {
                        if(params[k][1].equalsIgnoreCase("namespaceuri") ||
                           params[k][1].equalsIgnoreCase("uri"))
                           writer.print("qname.getNamespaceURI()");
                        else if(params[k][1].equalsIgnoreCase("localname") ||
                                params[k][1].equalsIgnoreCase("name"))
                           writer.print("qname.getLocalName()");
                        else
                           error("UnknownStringParameter",params[k][1]);
                     }
                     else
                        error("UnknownParameterType",params[k][0]);
                  }
                  writer.println(");");
                  writer.println("                     break;");
               }
            }
            writer.println("               }");
            writer.println("               break;");
         }
         writer.println("         }");
         writer.println("      }");
         writer.println("      catch(Exception e)");
         writer.println("      {");
         writer.println("         if(e instanceof SAXException)");
         writer.println("            throw (SAXException)e;");
         writer.println("         else");
         writer.println("            throw new HCException(e);");
         writer.println("      }");
         writer.println("   }");
         writer.println();
         writer.println("   public void acceptEndEvent(int xpath,int state,QName qname)");
         writer.println("      throws SAXException");
         writer.println("   {");
         writer.println("      try");
         writer.println("      {");
         writer.println("         switch(xpath)");
         writer.println("         {");
         for(int i = 0;i < tables.length;i++)
         {
            writer.print("            case ");
            writer.print(i);
            writer.println(':');
            writer.println("               switch(state)");
            writer.println("               {");
            for(int j = 0;j < tables[i].astate.length;j++)
            {
               MethodInfo method = (MethodInfo)tables[i].astate[j];
               if(method != null &&
                  (method.getName().startsWith("end") ||
                   method.getName().startsWith("close")))
               {
                  writer.print("                  case ");
                  writer.print(Integer.toString(j));
                  writer.println(':');
                  writer.print("                     handler.");
                  writer.print(method.getName());
                  writer.print('(');
                  String[][] params = method.getParameters();
                  for(int k = 0;k < params.length;k++)
                  {
                     if(k != 0)
                        writer.print(',');
                     if(params[k][0].equals("org.ananas.hc.QName"))
                        writer.print("qname");
                     else if(params[k][0].equals("java.lang.String"))
                     {
                        if(params[k][1].equalsIgnoreCase("namespaceuri") ||
                           params[k][1].equalsIgnoreCase("uri"))
                           writer.print("qname.getNamespaceURI()");
                        else if(params[k][1].equalsIgnoreCase("localname") ||
                                params[k][1].equalsIgnoreCase("name"))
                           writer.print("qname.getLocalName()");
                        else
                           error("UnknownStringParameter",params[k][1]);
                     }
                     else
                        error("UnknownParameterType",params[k][0]);
                  }
                  writer.println(");");
                  writer.println("                     break;");
               }
            }
            writer.println("               }");
            writer.println("               break;");
         }
         writer.println("         }");
         writer.println("      }");
         writer.println("      catch(Exception e)");
         writer.println("      {");
         writer.println("         if(e instanceof SAXException)");
         writer.println("            throw (SAXException)e;");
         writer.println("         else");
         writer.println("            throw new HCException(e);");
         writer.println("      }");
         writer.println("   }");
         writer.println();
         writer.println("   public void acceptCharactersEvent(int xpath,int state,char[] ch,int start,int length)");
         writer.println("      throws SAXException");
         writer.println("   {");
         writer.println("      try");
         writer.println("      {");
         writer.println("         switch(xpath)");
         writer.println("         {");
         for(int i = 0;i < tables.length;i++)
         {
            writer.print("            case ");
            writer.print(i);
            writer.println(':');
            writer.println("               switch(state)");
            writer.println("               {");
            for(int j = 0;j < tables[i].astate.length;j++)
            {
               MethodInfo method = (MethodInfo)tables[i].astate[j];
               if(method != null &&
                  (method.getName().startsWith("characters") ||
                   method.getName().startsWith("content")))
               {
                  writer.print("                  case ");
                  writer.print(Integer.toString(j));
                  writer.println(':');
                  writer.print("                     handler.");
                  writer.print(method.getName());
                  writer.print('(');
                  String[][] params = method.getParameters();
                  for(int k = 0;k < params.length;k++)
                  {
                     if(k != 0)
                        writer.print(',');
                     if(params[k][0].equals("char[]"))
                        writer.print("ch");
                     else if(params[k][0].equals("int"))
                     {
                        if(params[k][1].equalsIgnoreCase("start") ||
                           params[k][1].equalsIgnoreCase("offset"))
                           writer.print("start");
                        else if(params[k][1].equalsIgnoreCase("length") ||
                                params[k][1].equalsIgnoreCase("count") ||
                                params[k][1].equalsIgnoreCase("len"))
                           writer.print("length");
                        else
                           error("UnknownIntParameter",params[k][1]);
                     }
                     else if(params[k][0].equals("java.lang.String"))
                        writer.print("new String(ch,start,length)");
                     else
                        error("UnknownParameterType",params[k][0]);
                  }
                  writer.println(");");
                  writer.println("                     break;");
               }
            }
            writer.println("               }");
            writer.println("               break;");
         }
         writer.println("         }");
         writer.println("      }");
         writer.println("      catch(Exception e)");
         writer.println("      {");
         writer.println("         if(e instanceof SAXException)");
         writer.println("            throw (SAXException)e;");
         writer.println("         else");
         writer.println("            throw new HCException(e);");
         writer.println("      }");
         writer.println("   }");
         writer.println('}');
      }
      finally
      {
         writer.close();
      }
   }
}
