/**
 * This interface defines methods used by HC to report errors, warnings
 * and other information messages. UI should implement this interface
 * to report messages to the user.
 *
 * @author <a href="http://www.marchal.com">Beno&#238;t Marchal</a>
 */

// Copyright 2001-2002, Benot Marchal.

package org.ananas.hc.compiler;

public interface Messenger
{
   /**
    * Non-fatal error message.
    * @param e exception describing the error.
    */
   public void error(CompilerException e)
      throws CompilerException;

   /**
    * Fatal error message.
    * @param e exception describing the error.
    */
   public void fatal(CompilerException e)
      throws CompilerException;

   /**
    * Warning message. This is not an error.
    * @param e exception describing the warning.
    */
   public void warning(CompilerException e)
      throws CompilerException;

   /**
    * Catch-all for other messages. UI should display this to the user.
    * @param msg information message
    */
   public void info(String msg)
      throws CompilerException;
}
