package org.ananas.hc.test;

import java.io.*;
import java.util.*;
import org.xml.sax.*;
import org.ananas.hc.*;
import org.xml.sax.helpers.*;
import org.ananas.hc.compiler.*;

public class CompileTest
{
   protected static class DFAHandler
      extends DefaultHandler
   {
      protected DFATable table;
      protected int state;
      protected Stack stack;

      public DFAHandler(DFATable table)
      {
         this.table = table;
      }

      public void startDocument()
      {
         state = -1;
         stack = new Stack();
      }

      public void startElement(String namespaceURI,
                               String localName,
                               String qualifiedName,
                               Attributes atts)
      {
         stack.push(new Integer(state));
         QName qname = new QName(QName.ELEMENT,namespaceURI,localName);
         state = table.move(qname,state);
         System.out.println(qname.toString() + " caused a transition to " + state);
         if(table.isAcceptingState(state))
            System.out.println("found " + table.getAssociatedData(state));
      }

      public void endElement(String namespaceURI,
                             String localName,
                             String qualifiedName)
      {
         state = ((Integer)stack.pop()).intValue();
      }
   }

   private static void printDetails(XPathNode node,int inc)
   {
      if(node == null)
         return;
//       if(node.isQName())
      {
         for(int i = 0;i < inc;i++)
            System.out.print("  ");
         System.out.println(node.toString());
         for(int i = 0;i < inc;i++)
            System.out.print("  ");
         Iterator iterator = node.firstpos().iterator();
         System.out.print("( firstpos ");
         while(iterator.hasNext())
         {
            System.out.print(iterator.next().toString());
            System.out.print(' ');
         }
         System.out.println(')');
         for(int i = 0;i < inc;i++)
            System.out.print("  ");
         iterator = node.lastpos().iterator();
         System.out.print("( lastpost ");
         while(iterator.hasNext())
         {
            System.out.print(iterator.next().toString());
            System.out.print(' ');
         }
         System.out.println(')');
         for(int i = 0;i < inc;i++)
            System.out.print("  ");
         iterator = node.followpos().iterator();
         System.out.print("( followpos ");
         while(iterator.hasNext())
         {
            System.out.print(iterator.next().toString());
            System.out.print(' ');
         }
         System.out.println(')');
         printDetails(node.getLeft(),inc + 1);
         printDetails(node.getRight(),inc + 1);
      }
/*      else
         printDetails(node.getLeft(),inc);
         printDetails(node.getRight(),inc);*/
   }

   public static void main(String[] params)
      throws IOException, CompilerException, SAXException,
             ClassNotFoundException, IllegalAccessException,
             InstantiationException
   {
      NamespaceSupport namespace = new NamespaceSupport();
      namespace.declarePrefix("db","http://www.ananas.org/2001/docbook");
      MessageStore message = MessageStore.getMessageStore("org.ananas.hc.compiler.Message");
      XPathParser parser = new XPathParser(namespace,message);
/*      String xpath1 = "db:sect1/db:simpara/db:ulink",
             xpath2 = "db:sect1/db:sect1info/db:title";*/
      String xpath1 = "db:sect1/db:simpara/db:ulink",
             xpath2 = "db:sect1/db:simpara";
      XPathNode first = parser.axpath(xpath1,1,xpath1),
                second = parser.axpath(xpath2,2,xpath2),
                root = new XPathNode(XPathNode.OR_XPATH,first,second);
      DFAFactory factory = new DFAFactory();
      DFATable table = factory.createDFA(root);
/*      System.out.println("xpath:");
      System.out.println(root.toString());
      System.out.println("DFATable:");
      System.out.println(table.toString());*/
//      String xmlDoc = "<db:sect1 xmlns:db='http://www.ananas.org/2001/docbook'><db:sect1info><db:title/></db:sect1info></db:sect1>";
      String xmlDoc = "<db:sect1 xmlns:db='http://www.ananas.org/2001/docbook'><db:sect1info/><db:simpara/><db:simpara><db:ulink/></db:simpara><db:simpara><db:inlinegraphic/></db:simpara></db:sect1>";
      StringReader reader = new StringReader(xmlDoc);
      XMLReader xparser = XMLReaderFactory.createXMLReader("org.apache.xerces.parsers.SAXParser");
//      xparser.setContentHandler(new DFAHandler(table));
//      xparser.setContentHandler(new DFAHandler(new TestApplicationHandler__org_ananas_hc_tables()));
      xparser.setContentHandler(new XPathHandler(new TestApplicationHandler()));
      xparser.parse(new InputSource(reader));
/*      System.out.println("details:");
      printDetails(root,0);*/
   }
}
