package org.ananas.hc.test;

import junit.framework.TestCase;
import org.ananas.hc.compiler.XPathNode;

public class XPathNodeTest
   extends TestCase
{
   protected XPathNode absolute,
                       article,
                       simpara,
                       _simpara,
                       sect1,
                       _sect1,
                       href,
                       ulink,
                       root,
                       sect1Simpara,
                       _sect1Simpara,
                       rootSect1Simpara,
                       _rootSect1Simpara,
                       simparaHRef,
                       sect1SimparaOrSimparaHRef,
                       endMarker,
                       sect1SimparaOrSimparaHRefOrEndMarker;
   protected Object data1,
                    data2;
   protected XPathNode[] xpathnodes;
   protected String ns;
   private String[] strings;

   private XPathNode buildPath(XPathNode[] nodes)
   {
      XPathNode root = nodes[0];
      for(int i = 1;i < nodes.length;i++)
         root = new XPathNode(XPathNode.PARENT_OF,root,nodes[i]);
      return root;
   }

   public XPathNodeTest(String name)
   {
      super(name);
   }

   protected void setUp()
      throws Exception
   {
      init();
   }

   protected void init()
   {
      // StringBuffers enable testing internalization of Strings
      // otherwise "simpara" == "simpara"
      ns = "http://www.ananas.org/2001/docbook";
      StringBuffer uri = new StringBuffer("http://www.ananas.org/2001/docbook"),
                   tSimpara = new StringBuffer("simpara"),
                   tSect1 = new StringBuffer("sect1"),
                   empty = new StringBuffer("");
      simpara = new XPathNode(XPathNode.ELEMENT,
                              uri.toString(),
                              tSimpara.toString());
      _simpara = new XPathNode(XPathNode.ELEMENT,ns,"simpara");
      sect1    = new XPathNode(XPathNode.ELEMENT,null,tSect1.toString());
      _sect1   = new XPathNode(XPathNode.ELEMENT,uri.toString(),"sect1");
      absolute = new XPathNode(XPathNode.ROOT,null,"/");
      article  = new XPathNode(XPathNode.ELEMENT,ns,"article");
      sect1    = new XPathNode(XPathNode.ELEMENT,ns,"sect1");
      simpara  = new XPathNode(XPathNode.ELEMENT,ns,"simpara");
      _simpara = new XPathNode(XPathNode.ELEMENT,ns,"simpara");
      ulink    = new XPathNode(XPathNode.ELEMENT,ns,"ulink");
      href     = new XPathNode(XPathNode.ATTRIBUTE,null,"href");
      root     = new XPathNode(XPathNode.ROOT,null,"/");
      xpathnodes = new XPathNode[]
      {
         buildPath(new XPathNode[]
         {
            absolute, article, sect1, simpara
         }),
         buildPath(new XPathNode[]
         {
            sect1, ulink
         }),
         buildPath(new XPathNode[]
         {
            ulink, href
         }),
         buildPath(new XPathNode[]
         {
            absolute, article, sect1, simpara, ulink, href
         }),
      };
      strings = new String[]
      {
         "(/) / {http://www.ananas.org/2001/docbook}article / {http://www.ananas.org/2001/docbook}sect1 / {http://www.ananas.org/2001/docbook}simpara",
         "{http://www.ananas.org/2001/docbook}sect1 / {http://www.ananas.org/2001/docbook}ulink",
         "{http://www.ananas.org/2001/docbook}ulink / {}@href",
         "(/) / {http://www.ananas.org/2001/docbook}article / {http://www.ananas.org/2001/docbook}sect1 / {http://www.ananas.org/2001/docbook}simpara / {http://www.ananas.org/2001/docbook}ulink / {}@href",
      };
      sect1Simpara = new XPathNode(XPathNode.PARENT_OF,sect1,simpara);
      _sect1Simpara = new XPathNode(XPathNode.PARENT_OF,_sect1,_simpara);
      rootSect1Simpara = new XPathNode(XPathNode.PARENT_OF,root,sect1Simpara);
      _rootSect1Simpara = new XPathNode(XPathNode.PARENT_OF,root,_sect1Simpara);
      simparaHRef = new XPathNode(XPathNode.PARENT_OF,simpara,href);
      data1 = "data1";
      data2 = "data2";
      endMarker = new XPathNode(XPathNode.END_MARK);
      sect1SimparaOrSimparaHRef = new XPathNode(XPathNode.OR_XPATH,sect1Simpara,simparaHRef);
      sect1SimparaOrSimparaHRefOrEndMarker = new XPathNode(XPathNode.OR_XPATH,sect1SimparaOrSimparaHRef,endMarker);
   }

   public void testToString()
   {
      assertTrue(xpathnodes.length == strings.length);
      int count = 0;
      for(int i = 0;i < xpathnodes.length;i++)
         try
         {
            assertEquals(strings[i],xpathnodes[i].toString());
         }
         catch(Exception e)
         {
            count++;
         }
      assertTrue(count == 0);
   }

   public void testGetter()
   {
      assertEquals(rootSect1Simpara.getNamespaceURI(),"");
      assertEquals(rootSect1Simpara.getLocalName(),"");
      assertTrue(rootSect1Simpara.getType() == XPathNode.PARENT_OF);
      assertTrue(endMarker.getType() == XPathNode.END_MARK);
      assertEquals(rootSect1Simpara.getLeft(),root);
      assertEquals(rootSect1Simpara.getRight(),sect1Simpara);
      assertTrue(sect1SimparaOrSimparaHRef.getType() == XPathNode.OR_XPATH);
      assertEquals(sect1SimparaOrSimparaHRef.getLeft(),sect1Simpara);
      assertEquals(sect1SimparaOrSimparaHRef.getRight(),simparaHRef);
      assertEquals(sect1SimparaOrSimparaHRefOrEndMarker.getRight(),endMarker);
   }

   public void testEquals()
   {
      assertTrue(!sect1Simpara.equals(_sect1Simpara));
      assertTrue(!sect1.equals(_sect1));
      assertTrue(!sect1Simpara.equals(rootSect1Simpara));
      assertTrue(sect1Simpara.isSynonymousWith(_sect1Simpara,true));
      assertTrue(!sect1Simpara.isSynonymousWith(_sect1Simpara,false));
      assertTrue(rootSect1Simpara.isSynonymousWith(_rootSect1Simpara,true));
      assertTrue(!sect1Simpara.isSynonymousWith(rootSect1Simpara,true));
      assertTrue(!sect1Simpara.isSynonymousWith(rootSect1Simpara,false));
   }

	public static void main(String params[])
   {
		junit.awtui.TestRunner.run(XPathNodeTest.class);
	}
}
