package org.ananas.hc.test;

import junit.framework.TestCase;
import org.ananas.hc.MessageStore;
import org.ananas.hc.compiler.XPathNode;
import org.ananas.hc.compiler.Messenger;
import org.ananas.hc.compiler.XPathParser;
import org.xml.sax.helpers.NamespaceSupport;
import org.ananas.hc.compiler.CompilerException;

public class XPathParserTest
   extends XPathNodeTest
{
   private XPathNode absolute,
                     article,
                     sect1,
                     simpara,
                     _simpara,
                     ulink,
                     href;
   private String[] xpaths;
   private NamespaceSupport namespace;
   private MessageStore message;
   private XPathParser parser;

	public XPathParserTest(String name)
   {
		super(name);
	}

   protected void setUp()
      throws CompilerException
   {
      init();
      xpaths = new String[]
      {
         "/db:article/db:sect1/db:simpara",
         "db:sect1/db:ulink",
         "db:ulink/@href",
         "/db:article/db:sect1/db:simpara/db:ulink/@href",
      };
      namespace = new NamespaceSupport();
      namespace.declarePrefix("db",ns);
      message = MessageStore.getMessageStore("org.ananas.hc.compiler.Message");
      parser = new XPathParser(namespace,message);
   }

   private XPathNode buildPath(XPathNode[] nodes)
   {
      XPathNode root = nodes[0];
      for(int i = 1;i < nodes.length;i++)
         root = new XPathNode(XPathNode.PARENT_OF,root,nodes[i]);
      return root;
   }

   public void testParsing()
      throws CompilerException
   {
      assertTrue(xpaths.length == xpathnodes.length);
      for(int i = 0;i < xpaths.length;i++)
         assertTrue(parser.xpath(xpaths[i]).isSynonymousWith(xpathnodes[i],true));
   }

   public void testInvalidXPathParsing()
   {
      String[] invalids = new String[]
      {
         "db:simpara/db:ulink[@href]",
         "db:sect1/sm:simpara",
         "db:sect1//db:ulink"
      };
      int count = 0;
      for(int i = 0;i < invalids.length;i++)
         try
         {
            parser.xpath(invalids[i]);
         }
         catch(CompilerException e)
         {
            count++;
         }
      assertTrue(invalids.length == count);
   }

	public static void main(String params[])
   {
		junit.awtui.TestRunner.run(XPathParserTest.class);
	}
}
